/* 
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <libgen.h>
#include <stdio.h>
#include <errno.h>
#include <getopt.h>
#include <sys/stat.h>
#include <string.h>
#include "va_log.h"
#include "va_assert.h"
#include "varid.h"
#include "config.h"
#include <float.h>
#include <sys/types.h>
#include <fcntl.h>
#include <omp.h>

char state_list[40][2] = 
    {
        /* ll_list[16][2] */ 
        {'a', 'a'}, {'a', 'c'}, {'a', 'g'}, {'a', 't'},
        {'c', 'a'}, {'c', 'c'}, {'c', 'g'}, {'c', 't'},
        {'g', 'a'}, {'g', 'c'}, {'g', 'g'}, {'g', 't'},
        {'t', 'a'}, {'t', 'c'}, {'t', 'g'}, {'t', 't'},
 
        /* ld_list[4][2] */ 
        {'a', 'A'}, {'c', 'C'}, {'g', 'G'}, {'t', 'T'},

        /* dd_list[4][2] */ 
        {'A', 'A'}, {'C', 'C'}, {'G', 'G'}, {'T', 'T'},

        /* dl_list[16][2] */ 
        {'A', 'a'}, {'A', 'c'}, {'A', 'g'}, {'A', 't'},
        {'C', 'a'}, {'C', 'c'}, {'C', 'g'}, {'C', 't'},
        {'G', 'a'}, {'G', 'c'}, {'G', 'g'}, {'G', 't'},
        {'T', 'a'}, {'T', 'c'}, {'T', 'g'}, {'T', 't'}
    };

#define	DEF_NO_HEURISTICS  1
#define DEF_GAP_ERROR	0.03
#define DEF_SNP_RATE	0.001
#define DEF_CALL_THRESHOLD	0.75
#define DEF_LOG_CUTOFF		5000.0
#define DEF_INSERT_OPEN		0.0001
#define DEF_INSERT_EXTEND	0.001
#define DEF_DELETION_OPEN	0.0001
#define DEF_DELETION_EXTEND	0.001
#define DEF_GET_BP_FREQ		FALSE
#define DEF_MIN_COVERAGE	4
#define DEF_FREQ_A		0.25
#define DEF_FREQ_C		0.25
#define DEF_FREQ_G		0.25
#define DEF_FREQ_T		0.25
#define DEF_ERR_COVERAGE	40
#define DEF_CS_ERR		0.04
#define DEF_LS_ERR		0.01
#define DEF_SAVE_LEARNED_FILE	NULL
#define DEF_LOAD_LEARNED_FILE	NULL
#define DEF_MIN_CS_ERR		0.001
#define DEF_MAX_CS_ERR		0.99
#define DEF_MIN_LS_ERR		0.001
#define DEF_MAX_LS_ERR		0.99
#define DEF_TEMP_DIRECTORY	"./varid_tmp"
#define DEF_DONOR_NAME	"DONOR_GENOME"
#define DEF_ALIGNMENTS_FILENAME	"stdin" 
#define DEF_VITERBI	0
#define DEF_BINOMIAL	0
#define DEF_POISSON	0
#define DEF_NORMAL	0
#define DEF_RANK_SUM	1
#define DEF_LOG_COEFFICIENT_LS	1
#define DEF_LOG_COEFFICIENT_CS	1
#define DEF_HAPLOID	0
#define DEF_DROP_BAD_READS 0
#define DEF_NUMBER_OF_THREADS 1
#define DEF_HMM_AREA	200
#define DEF_NORMAL_STD	1.5
#define DEF_FORMAT	FORMAT_PB
#define DEF_REPORT_ALL	0

int no_heuristics	= DEF_NO_HEURISTICS;
double gap_error	= DEF_GAP_ERROR;
double snp_rate 	= DEF_SNP_RATE;
double call_threshold 	= DEF_CALL_THRESHOLD;
double log_cutoff	= DEF_LOG_CUTOFF;
int get_bp_freq		= DEF_GET_BP_FREQ;
double freq[] = {DEF_FREQ_A,DEF_FREQ_C,
		 DEF_FREQ_G,DEF_FREQ_T};
int min_coverage	= DEF_MIN_COVERAGE;
int err_coverage	= DEF_ERR_COVERAGE;
double cs_err		= DEF_CS_ERR;
double ls_err		= DEF_LS_ERR;
FILE * save_learned_file	= DEF_SAVE_LEARNED_FILE;
FILE * load_learned_file	= DEF_LOAD_LEARNED_FILE;
double min_cs_err	= DEF_MIN_CS_ERR;
double max_cs_err	= DEF_MAX_CS_ERR;
double min_ls_err	= DEF_MIN_LS_ERR;
double max_ls_err	= DEF_MAX_LS_ERR;
double insert_open	= DEF_INSERT_OPEN;
double insert_extend	= DEF_INSERT_EXTEND;
double deletion_open	= DEF_DELETION_OPEN;
double deletion_extend	= DEF_DELETION_EXTEND;
int viterbi	= DEF_VITERBI;
long long predict_start	= 0;
long long predict_end	= -1;
long long hmm_start	= 0;
long long hmm_end	= -1;
int poisson	= DEF_POISSON;
int normal	= DEF_NORMAL;
int rank_sum	= DEF_RANK_SUM;
int binomial	= DEF_BINOMIAL;
double normal_std = DEF_NORMAL_STD;
double log_coefficient_ls	= DEF_LOG_COEFFICIENT_LS;
double log_coefficient_cs	= DEF_LOG_COEFFICIENT_CS;
int haploid	= DEF_HAPLOID;
int drop_bad_reads = DEF_DROP_BAD_READS;
int number_of_threads = DEF_NUMBER_OF_THREADS;
int hmm_area = DEF_HMM_AREA;
int format = DEF_FORMAT;
int report_all = DEF_REPORT_ALL;
static void parse_args(
    int    argc,
    char * argv[]);

static void print_usage(
    int          exit_code,
    const char * s,
    int		full);
char * error_strings[] = { "A read falls off the end of the genome! Please remove it!", 
                           "Error parsing the quality values or position specific error values for read", 
                           "Internal error PR001", 
                           "Internal error PR002", 
                           "Internal error PR003", 
                           "Internal error PR004", 
                           "Internal error PR005", 
                            };  


char * alignments_filename = DEF_ALIGNMENTS_FILENAME;
FILE * alignments_file = NULL;
char * reference_filename = NULL;
FILE * reference_file = NULL;
char * donor_name = DEF_DONOR_NAME;
FILE * output_file = NULL;
char * temp_directory = DEF_TEMP_DIRECTORY;


/*Probably there is a better place for these, but I don't know it */
double*  cs_error_cnt;
double*  cs_error_dpts;
unsigned int cs_error_size;
double*  ls_error_cnt;
double*  ls_error_dpts;
unsigned int ls_error_size;

/* program name */
static char * progName = NULL;

/* Called upon application exit */
static void varid_exit(void);


int main(
    int    argc, 
    char * argv[])
{
    output_file = stdout;
    alignments_file = stdin;
    alignments_filename = "stdin"; 
    progName = argv[0];

    cs_error_cnt = (double*)malloc(sizeof(double)*MAX_QVAL);
    VA_ASSERT(cs_error_cnt!=NULL);
    memset(cs_error_cnt,0,sizeof(double)*MAX_QVAL);
    cs_error_dpts = (double*)malloc(sizeof(double)*MAX_QVAL);
    VA_ASSERT(cs_error_dpts!=NULL);
    memset(cs_error_dpts,0,sizeof(double)*MAX_QVAL);
    cs_error_size = MAX_QVAL;
    ls_error_cnt = (double*)malloc(sizeof(double)*MAX_QVAL);
    VA_ASSERT(ls_error_cnt!=NULL);
    memset(ls_error_cnt,0,sizeof(double)*MAX_QVAL);
    ls_error_dpts = (double*)malloc(sizeof(double)*MAX_QVAL);
    VA_ASSERT(ls_error_dpts!=NULL);
    memset(ls_error_dpts,0,sizeof(double)*MAX_QVAL);
    ls_error_size=MAX_QVAL;
    parse_args(argc, argv);
  

    /* Open the syslog */
    LOG_OPEN(basename(progName));

    /* Register a function to be called when the application exit()s */
    (void)atexit(varid_exit);

    varid_single();
    if (output_file!=stdout) {
	fclose(output_file);	
    }
    fclose(reference_file);
    if (alignments_file!=stdin) {
	fclose(alignments_file);
    }


    exit (EXIT_SUCCESS);
}

static void parse_args(
    int    argc,
    char * argv[])
{
    char short_op[] = "hva:r:n:o:d:eg:s:t:l:S:L:";
    struct option long_op[] =
    {
        {"help", 0, 0, 'h'},
        {"version", 0, 0, 'v'},
        {"alignments", 1, 0, 'a'},
        {"ref-file", 1, 0, 'r'},
        {"donor-name", 1, 0, 'n'},
        {"output", 1, 0, 'o'},
        {"temp-dir", 1, 0, 'd'},
	{"no-heuristics",0,0,'e'},
	{"gap-error",1,0,'g'},
	{"snp-rate",1,0,'s'},
	{"call-threshold",1,0,'t'},
	{"log-cutoff",1,0,'l'},
	{"deletion-open",1,0,4},
	{"deletion-extend",1,0,5},
	{"insert-open",1,0,6},
	{"insert-extend",1,0,7},
	{"get-bp-freq",0,0,8},
	{"freq-a",1,0,9},
	{"freq-c",1,0,10},
	{"freq-g",1,0,11},
	{"freq-t",1,0,12},
	{"min-coverage",1,0,13},
	{"err-coverage",1,0,14},
	{"ls-err",1,0,15},
	{"cs-err",1,0,16},
	{"save-learned",1,0,'S'},
	{"load-learned",1,0,'L'},
	{"min-cs-err",1,0,19},
	{"max-cs-err",1,0,20},
	{"min-ls-err",1,0,21},
	{"max-ls-err",1,0,22},
	{"predict-start",1,0,23},
	{"predict-end",1,0,24},
	{"viterbi",0,0,25},
	{"poisson",0,0,26},
	{"normal",1,0,27},
	{"log-coeff-ls",1,0,28},
	{"log-coeff-cs",1,0,29},
	{"haploid",0,0,30},
	{"drop-bad-reads",0,0,31},
	{"threads",1,0,32},
	{"hmm-start",1,0,33},
	{"hmm-end",1,0,34},
	{"hmm-area",1,0,35},
	{"rank-sum",1,0,36},
	{"binomial",0,0,37},
	{"format",1,0,38},
	{"all",0,0,39},
        {0, 0, 0, 0}
    };
    int op_id;
    char c;
    char * save_learned_filename=NULL; char * load_learned_filename=NULL; 
    c = getopt_long(argc, argv, short_op, long_op, &op_id);
    int check_ranges=0;
    while (c != EOF) {
        switch (c) {
	    case 4:
		deletion_open=atof(optarg);
		fprintf(stderr, "Using reference deletion open probability %e\n",deletion_open);
		break;
	    case 5:
		deletion_extend=atof(optarg);
		fprintf(stderr, "Using reference deletion extend probability %e\n",deletion_extend);
		break;
	    case 6:
		insert_open=atof(optarg);
		fprintf(stderr, "Using reference insert open probability %f\n",insert_open);
		break;
	    case 7:
		insert_extend=atof(optarg);
		fprintf(stderr, "Using reference insert extend probability %f\n",insert_extend);
		break;
	    case 8:
		get_bp_freq=TRUE;
		fprintf(stderr, "Guessing bp freq based on reference\n");
		break;
	    case 9:
		freq[0]=atof(optarg);
		fprintf(stderr, "Using %e for frequency of bp a\n",freq[0]);
		break;
	    case 10:
		freq[1]=atof(optarg);
		fprintf(stderr, "Using %e for frequency of bp c\n",freq[1]);
		break;
	    case 11:
		freq[2]=atof(optarg);
		fprintf(stderr, "Using %e for frequency of bp g\n",freq[2]);
		break;
	    case 12:
		freq[3]=atof(optarg);
		fprintf(stderr, "Using %e for frequency of bp t\n",freq[3]);
		break;
	    case 13:
		min_coverage=atoi(optarg);
		fprintf(stderr, "Using %d for min coverage to call\n",min_coverage);
		break;
	    case 14:
		err_coverage=atoi(optarg);
		fprintf(stderr, "Using %d for min error estimate coverage\n",err_coverage);
		break;
	    case 15:
		ls_err=atof(optarg);
		fprintf(stderr, "Using %e for default letter space error rate\n",ls_err);
		break;  
	    case 16:
		cs_err=atof(optarg);
		fprintf(stderr, "Using %e for default colour space error rate\n",cs_err);
		break;  
	    case 'S':
		save_learned_filename = optarg;
		save_learned_file = fopen(save_learned_filename, "w");
		if (save_learned_file == NULL) {
			perror(" Failed to open file to save to. Exiting...");
			exit(1);
		}
		fprintf(stderr, "Using \"%s\" as a file to save learned error rates to\n",save_learned_filename);
		break;  
	    case 'L':
		load_learned_filename = optarg;
		load_learned_file = fopen(load_learned_filename,"r");
		if (load_learned_file == NULL) {
			perror(" Failed to open file to load. Exiting...");
			exit(1);
		}
		fprintf(stderr, "Using \"%s\" as a file to load learned error rates from\n",load_learned_filename);
		break;
	    case 19:
		min_cs_err=atof(optarg);
		fprintf(stderr, "Using %e as minimum color space read error\n",min_cs_err);
		break;
	    case 20:
		max_cs_err=atof(optarg);
		fprintf(stderr, "Using %e as maximum color space read error\n",max_cs_err);
		break;
	    case 21:
		min_ls_err=atof(optarg);
		fprintf(stderr, "Using %e as minimum letter space read error\n",min_ls_err);
		break;
	    case 22:
		max_ls_err=atof(optarg);
		fprintf(stderr, "Using %e as maximum letter space read error\n",max_ls_err);
		break;
	    case 23:
		predict_start=atoi(optarg);
		if (predict_start<=0) {
			fprintf(stderr,"predict_start must be a positive integer!\n");
			print_usage(1,argv[0],0);
		}
		fprintf(stderr, "Using %lld as prediction start index\n",predict_start);
		check_ranges=1;
		break;
	    case 24:
		predict_end=atoi(optarg);
		if (predict_end<=0) {
			fprintf(stderr,"predict_start must be a positive integer!\n");
			print_usage(1,argv[0],0);
		}
		fprintf(stderr, "Using %lld as prediction end index\n",predict_end);
		check_ranges=1;
		break;
	    case 25:
		viterbi=1;
		fprintf(stderr, "Using viterbi prediction algorithm instead of forward / backward\n");
		break; 
	    case 26:
		poisson=1;
		binomial=0; rank_sum=0; normal=0;
		fprintf(stderr, "Using poisson for coverage distribution\n");
		break;
	    case 27:
		normal=1;
		binomial=0; poisson=0; rank_sum=0;
		normal_std=atof(optarg);
		fprintf(stderr, "Using normal for coverage distribution with std %e\n",normal_std);
		break; 
	    case 28:
		log_coefficient_ls=atof(optarg);
		fprintf(stderr, "Scaling log emissions of letter signal by %e\n",log_coefficient_ls);
		break;
	    case 29:
		log_coefficient_cs=atof(optarg);
		fprintf(stderr, "Scaling log emissions of color signal by %e\n",log_coefficient_cs);
		break;
	    case 30:
		haploid=1;
		fprintf(stderr, "Assuming single ploidy of donor organism\n");
		break;
	    case 31:
		drop_bad_reads=1;
		fprintf(stderr, "Dropping bad reads, instead of exiting on bad reads\n");
		break;
	    case 32:
		number_of_threads=atoi(optarg);
		if (number_of_threads<1) {
			fprintf(stderr,"threads parameter must be a positive integer!\n");
			print_usage(1,argv[0],0);
		}
		fprintf(stderr, "Using %d threads\n",number_of_threads);
		break;
	    case 33:
		hmm_start=atoi(optarg);
		if (hmm_start<=0) {
			fprintf(stderr,"hmm_start must be a positive integer!\n");
			print_usage(1,argv[0],1);
		}
		check_ranges=1;
		fprintf(stderr, "Using %lld as hmm_start index\n",hmm_start);
		break;
	    case 34:
		hmm_end=atoi(optarg);
		if (hmm_end<=0) {
			fprintf(stderr,"hmm_start must be a positive integer!\n");
			print_usage(1,argv[0],1);
		}
		check_ranges=1;
		fprintf(stderr, "Using %lld as hmm_end index\n",hmm_end);
		break;
	    case 35:
		hmm_area=atoi(optarg);
		if (hmm_area<=0) {
			fprintf(stderr,"hmm_area must be a positive integer!\n");
			print_usage(1,argv[0],1);
		}
		check_ranges=1;
		fprintf(stderr, "Using %d as hmm_area index\n",hmm_area);
		break;
	    case 36:
		rank_sum=1;
		binomial=0; poisson=0; normal=0;
		normal_std=atof(optarg);
		fprintf(stderr,"Using rank sum for distribution penalty\n");
		break;
	    case 37:
		binomial=1;
		rank_sum=0; poisson=0; normal=0;
		fprintf(stderr,"Using binomial for distribution penalty\n");
		break;
	    case 38:
		if (strcmp(optarg,"vcf")==0 || strcmp(optarg,"VCF")==0) {
			fprintf(stderr,"Using VCF as output format!\n");
			format=FORMAT_VCF;
		} else if (strcmp(optarg,"pb")==0 || strcmp(optarg,"PB")==0) {
			fprintf(stderr,"Using pretty base as output format!\n");
			format=FORMAT_PB;
		} else {
			fprintf(stderr,"Please use VCF of PB as output format!\n");
			exit(1);
		}
		break;
	    case 39:
		report_all=1;
		break;
	    case 'g':
		gap_error=atof(optarg);
	        fprintf(stderr, "Using gap_error %e\n", gap_error);
	        break;
	    case 's':
		snp_rate=atof(optarg);
	        fprintf(stderr, "Using snp_rate %e\n", snp_rate);
	        break;
            case 'h': /* help */
		print_usage(1,argv[0],1);
                break;
            case 'v': /* version */
                printf ("VARiD %s\n", PACKAGE_VERSION);
                exit (EXIT_SUCCESS);
                break;

            case 'a': /* reads file */
		alignments_filename = strdup(optarg);
		alignments_file = fopen(alignments_filename,"r");
		if (alignments_file == NULL) {
			perror("Failed to open alignments file for reading");
		}
                break;

            case 'r': /* reference file */
		reference_filename = strdup(optarg);
		reference_file = fopen(reference_filename,"r");
		if (reference_file == NULL) {
			perror("Failed to open reference file for reading");
		}
                break;
 
            case 'n': /* donor name */
		donor_name=strdup(optarg);
                break;

            case 'o': /* write pretty file */
		output_file=fopen(optarg,"w");
		if (output_file==NULL) {
			perror("Failed to open file for writting output!");
			exit(1);
		}
                break;

            case 'd': /* temp-dir */
		temp_directory=strdup(optarg);
                break;
	    case 'e': /* enable heuristics */
		no_heuristics=FALSE;
		break;	
	    case 't':
	        call_threshold = atof(optarg);
	        fprintf(stderr, "Using optional threshold %e\n", call_threshold);
	        break;
	    case 'l':
		log_cutoff=atof(optarg);
		fprintf(stderr, "Using optional log-cutoff %e\n",log_cutoff);
		break;

            default: /* unknown option */
		print_usage(1,argv[0],1);
                break;

        }
        c = getopt_long(argc, argv, short_op, long_op, &op_id);
    }
    if (save_learned_file!=NULL && load_learned_file!=NULL) {
	fprintf(stderr,"Only one of --save-learned or --load-learned can be specified at a given time\n");
	exit(1);
    }
    if (binomial+poisson+rank_sum+normal!=1) {
	fprintf(stderr,"VARiD can run with only 1 type of distribution penalty selected at a time!\n");
	exit(1);
    }
    //make the temp folder
    struct stat fstat;
    if (stat(temp_directory, &fstat) == -1 ) {
    	//int res = mkdir(temp_directory,S_IRUSR | S_IWUSR | S_IRGRP | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
    	int res = mkdir(temp_directory,0777);
	if (res!=0) {
		perror("Failed to create a temporary directory for VARiD");
		exit(1);
	}
    }
    if (reference_file==NULL) {
	print_usage(1,argv[0],0);
    }
    omp_set_num_threads(number_of_threads);
    if (check_ranges==1) {
	if (predict_start<=0) {
		fprintf(stderr,"predict_start has not been set!\n");
		print_usage(1,argv[0],1);
	}
	if (predict_end<=0) {
		fprintf(stderr,"predict_end has not been set!\n");
		print_usage(1,argv[0],1);
	}
	if (hmm_start==0 && hmm_end==-1) {
		//hmm start and end have not been set, so set it
		hmm_start=MAX(1,predict_start-hmm_area);
		hmm_end=predict_end+hmm_area;
	}
	if (hmm_start>predict_start || hmm_end<predict_end || predict_start>predict_end || hmm_start>hmm_end) {
		fprintf(stderr,"please check that predict_start, predict_end, hmm_start, hmm_end are reasonable!\n");
		print_usage(1,argv[0],1);
	}
    }
}

static void print_usage(
    int          exit_code,
    const char * s,
    int		full)
{
    FILE * f;
    if (exit_code != 0)
        f = stderr;
    else
        f = stdout;

    if (s != NULL)
        fprintf(f, "%s: %s\n\n", progName, s);

   fprintf(f,"Usage: %s [Options] [Required Parameters]\n",progName); 
   fprintf(f,"Version: " PACKAGE_VERSION "\n");
  
fprintf(f,
    "Required Parameters:\n");
fprintf(f,
    "    -r/--ref-file         reference file name            \n");
fprintf(f,
    "Optional Parameters:\n");
fprintf(f,
    "    -a/--alignments       alignments file name                (default: %8s)\n",DEF_ALIGNMENTS_FILENAME);
fprintf(f,
    "    -n/--donor-name       donor name                          (default: \"%8s\")\n",DEF_DONOR_NAME);
fprintf(f,
    "    -d/--temp-dir         temporary folder                    (default: %8s)\n",DEF_TEMP_DIRECTORY);
fprintf(f,
    "    -t/--call-threshold   p threshold for calling a base      (default: %8.5f)\n",DEF_CALL_THRESHOLD);
fprintf(f,
    "       --min-coverage     min. coverage for calling a base    (default: %8d)\n",DEF_MIN_COVERAGE);
fprintf(f,
    "    -o/--output           write pretty file                   (default:   stdout)\n");
fprintf(f,
    "       --format [pb|vcf]  specify output format               (default:       pb)\n");
fprintf(f,
    "       --all              report calls for every base         (default: disabled)\n");
fprintf(f,
    "Optimizations:\n");
if (full==1) {
fprintf(f,
    "    -l/--log-cutoff       max log diff to consider in sums    (default: %8.2f)\n",DEF_LOG_CUTOFF);
}
fprintf(f,
    "       --threads          number of threads to use,see README (default: %8d)\n",1);
fprintf(f,
    "Probability settings:\n"); 
fprintf(f,
    "    -s/--snp-rate         probability of SNP                  (default: %8.5f)\n",DEF_SNP_RATE);
fprintf(f,
    "    -g/--gap-error        probability of gap error in read    (default: %8.5f)\n",DEF_GAP_ERROR);
fprintf(f,
    "       --insert-open      reference insert open probability   (default: %8.5f)\n",DEF_INSERT_OPEN);
fprintf(f,
    "       --insert-extend    reference insert extend probability (default: %8.5f)\n",DEF_INSERT_EXTEND);
fprintf(f,
    "       --deletion-open    reference deletion open prob.       (default: %8.5f)\n",DEF_DELETION_OPEN);
fprintf(f,
    "       --deletion-extend  reference deletion extend prob.     (default: %8.5f)\n",DEF_DELETION_EXTEND);
fprintf(f,
    "       --err-coverage     min coverage to estimate read error (default: %8d)\n",DEF_ERR_COVERAGE);
if (full==1) {
fprintf(f,
    "       --ls-err           default letter space read error     (default: %8.5f)\n",DEF_LS_ERR);
fprintf(f,
    "       --min-ls-err       the minimum letter space read error (default: %8.5f)\n",DEF_MIN_LS_ERR);
fprintf(f,
    "       --max-ls-err       the maximum letter space read error (default: %8.5f)\n",DEF_MAX_LS_ERR);
fprintf(f,
    "       --cs-err           default colour space read error     (default: %8.5f)\n",DEF_CS_ERR);
fprintf(f,
    "       --min-cs-err       the minimum colour space read error (default: %8.5f)\n",DEF_MIN_LS_ERR);
fprintf(f,
    "       --max-cs-err       the maximum colour space read error (default: %8.5f)\n",DEF_MAX_LS_ERR);
fprintf(f,
    "       --get-bp-freq      compute bp frequencies from reference\n"); 
fprintf(f,
    "       --freq-a           frequency of bp \"A\" in genome       (default: %8.5f)\n",DEF_FREQ_A);
fprintf(f,
    "       --freq-c           frequency of bp \"C\" in genome       (default: %8.5f)\n",DEF_FREQ_C);
fprintf(f,
    "       --freq-g           frequency of bp \"G\" in genome       (default: %8.5f)\n",DEF_FREQ_G);
fprintf(f,
    "       --freq-t           frequency of bp \"T\" in genome       (default: %8.5f)\n",DEF_FREQ_T);
fprintf(f,
    "       --log-coeff-ls     scale letter space log emissions    (default: %8d)\n",DEF_LOG_COEFFICIENT_LS);
fprintf(f,
    "       --log-coeff-cs     scale color space log emissions     (default: %8d)\n",DEF_LOG_COEFFICIENT_CS);
fprintf(f,
    "    -S/--save-learned     save learned read error then exit\n");     
fprintf(f,
    "    -L/--load-learned     load learned read error from file\n");    
}
fprintf(f,
    "HMM options:\n");
fprintf(f,
    "       --predict-start    predictions from this ref. index    (defualt: %8d)\n",1);
fprintf(f,
    "       --predict-end      predictions to this ref. index      (default: len(ref))\n");
if (full==1) {
fprintf(f,
    "       --hmm-start        start HMM model from this index     (default: %8d)\n",1);
fprintf(f,
    "       --hmm-end          end HMM model at this index         (default: len(ref))\n");
fprintf(f,
    "       --hmm-area         use hmm-area around predict region  (default: %8d)\n",DEF_HMM_AREA);
fprintf(f,
    "       --viterbi          use viterbi prediction algorithm\n");
}
fprintf(f,
    "Heterozygous distribution penalties:\n");
fprintf(f,
    "       --rank-sum         use rank sum with normal            (default: %s)\n",DEF_RANK_SUM==1 ? "enabled" : "disabled");
fprintf(f,
    "       --normal std       use normal with given std           (default: %s)\n",DEF_NORMAL==1 ? "enabled" : "disabled");
fprintf(f,
    "       --poisson          use poisson                         (default: %s)\n",DEF_POISSON==1 ? "enabled" : "disabled");
fprintf(f,
    "       --binomial         use binomial                        (default: %s)\n",DEF_BINOMIAL==1 ? "enabled" : "disabled");
fprintf(f,
    "Options:\n");
fprintf(f,
    "       --haploid          assume donor organism is haploid\n");
fprintf(f,
    "    -e/--heuristics       enable post processing heuristics   (default: %s)\n",DEF_NO_HEURISTICS==1 ? "disabled" : "enabled");
fprintf(f,
    "       --drop-bad-reads   drop reads that cannot be properly parsed\n");
fprintf(f,
    "    -v/--version          output version string\n");
fprintf(f,
    "    -h/--help             full usage\n");


   fprintf(f,"Contact <varid at cs dot toronto dot edu>.\n\n");
   fprintf(f,"Copyright (C) 2010\n");
   fprintf(f,"This program comes with ABSOLUTELY NO WARRANTY;\n");
   fprintf(f,"This is free software, and you are welcome to redistribute it\n");
   fprintf(f,"under certain conditions.\n\nAuthor: Adrian V. Dalca\n");
    
    exit (exit_code);
}

static void varid_exit(
    void)
{
    LOG_INFO1("%s exiting...", progName);    

    /* close syslog */
    LOG_CLOSE(progName);
}

