/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "va_assert.h"
#include "va_log.h"
#include "varid.h"

/******************************************************************************
* GET_COMPLEMENT returns the complement of str1
*   str2 = GET_COMPLEMENT(str1) returns the DNA complement of str1, where
*   str1 is given in terms of numbers 0-3 or letters a,c,g,t.
*
*   Parameter
*   str1 - the string to be complemented, in ns (0-3) or ls (a,c,g,t);
*
*   Returns
*   str2 - the complement of str1, in the same space (ns or ls)
*
*   See also NUC2NUC
*
*   Author: Adrian V. Dalca
******************************************************************************/
VA_VecU8 get_complement(
    VA_VecU8 str1)
{
    VA_VecU8 str2;
    int i;
    int isalnum = TRUE;

    /* if the input is in character form, assume it is a string of a,c,g and
     * t. Otherwise, assume it is a set of 1,2,3 and 4.
     */

    VA_ASSERT((str1.data != NULL) && (str1.size > 0));
    str2.size = str1.size;
    str2.data = (VA_UINT8 *)malloc(sizeof(VA_UINT8) * str1.size);
    VA_ASSERT(str2.data != NULL);
   
    for (i = 0; i < str1.size; i++)
    {
        if (!isalpha(str1.data[i]) && str1.data[i]!='-')
        {
            isalnum = FALSE;
            break;
        }
    }

    if (isalnum)
    {
        memset(str2.data, '.', str2.size * sizeof(VA_UINT8));
        for (i = 0; i < str1.size; i++)
        {
            VA_UINT8 ch = tolower(str1.data[i]);

            if (ch == 'a')
                str2.data[i] = 't';
            else if (ch == 'c')
                str2.data[i] = 'g';
            else if (ch == 'g')
                str2.data[i] = 'c';
            else if (ch == 't')
                str2.data[i] = 'a';
            else if (ch == '-')
                str2.data[i] = '-';
            else if (ch == 'n')
                str2.data[i] = 'n';
            else
            {
                LOG_ERROR1("%02x : str1 should contain "
                     "only a, c, g, t and -; or 0, 1, 2 3 and 4", str1.data[i]);
                exit(EXIT_FAILURE);
            }
        }
    }
    else
    {
        /* assuming a number format */
        memset(str2.data, (VA_UINT8)-1, str2.size * sizeof(VA_UINT8));
        for (i = 0; i < str1.size; i++)
        {
            if (str1.data[i] == 0)
                str2.data[i] = 3;
            else if (str1.data[i] == 1)
                str2.data[i] = 2;
            else if (str1.data[i] == 2)
                str2.data[i] = 1;
            else if (str1.data[i] == 3)
                str2.data[i] = 0;
            else if (str1.data[i] == 4)
                str2.data[i] = 4;
            else if (str1.data[i] == 5)
                str2.data[i] = 5;
            else
            {
                LOG_ERROR1("%02x : str1 should contain "
                     "only a, c, g, t and -; or 0, 1, 2 3 and 4", str1.data[i]);
                exit(EXIT_FAILURE);
            }
        }
    }

    return (str2);
}
