/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "va_log.h"
#include "va_assert.h"
#include "varid.h"

/******************************************************************************
* NUC2NUC translates between ns and ls formats for DNA. 
*   outnuc = NUC2NUC(innuc) translates nucleotide strands between ns (0-5)
*   and ls (a,c,g,t,-,n). If innuc is supplied in ns the translation is in 
*   ls and vice-versa. ns strands are supplied as a vector and ls strands 
*   are supplied as strings (vector of chars). NOTE: ls sequences have to
*   be given as lowercase. Any uppercase A, C, G ot T is considered a gap.
*
*   Paramter:
*   innuc - the inpus string, in ns or ls.
*
*   Returns:
*   outnuc - the resulting string
*
*   See also COL2NUC, NUC2COL, GET_COMPLEMENT
*
*   Author Adrian V. Dalca
******************************************************************************/
VA_VecU8 nuc2nuc(
    VA_VecU8 innuc)
{
    VA_VecU8 outnuc;
    int i;

    VA_ASSERT((innuc.data != NULL) && (innuc.size > 0));
  
    outnuc = va_allocVecU8(innuc.size);
    
    for (i = 0; i < innuc.size; i++)
    {
        /* ns 2 ls translation */
        if (isalpha(innuc.data[i]) || 
            (innuc.data[i] == '-') || (innuc.data[i] == 'n'))
        {
            outnuc.data[i] = 0;
            switch (innuc.data[i])
            {
            case 'a':
                outnuc.data[i] = 0;
                break;
            case 'c':
                outnuc.data[i] = 1;
                break;
            case 'g':
                outnuc.data[i] = 2;
                break;
            case 't':
                outnuc.data[i] = 3;
                break;
            case 'A':
            case 'C':
            case 'G':
            case 'T':
                outnuc.data[i] = 4;
                break;
            case '-':
                outnuc.data[i] = 4;
                break;
            case 'n':
                outnuc.data[i] = 5;
	    case '+':
		outnuc.data[i] = 6;
                break;
            
            default:
                LOG_ERROR1("Invalid innuc <%d>", innuc.data[i]);
		outnuc.data[i] = 5;
                //exit(EXIT_FAILURE);
            }
        }    
        else
        {
            /* ls 2 nr translation */    
            outnuc.data[i] = ' ';
            switch (innuc.data[i])
            {
            case 0:
                outnuc.data[i] = 'a';
                break;
            case 1:
                outnuc.data[i] = 'c';
                break;
            case 2:
                outnuc.data[i] = 'g';
                break;
            case 3:
                outnuc.data[i] = 't';
                break;
            case 4:
                outnuc.data[i] = '-';
                break;
            case 5:
                outnuc.data[i] = 'n';
                break;
	    case 6:
		outnuc.data[i] = '+';
            default:
                LOG_ERROR1("Invalid innuc <%d>", innuc.data[i]);
                exit(EXIT_FAILURE);
            }
        }
    }

    return (outnuc);
}
